import maya.cmds as cmds
import hmScripts.hm_global_variables as global_vars
import hmScripts.hm_global_variables as nc
from hmScripts.hm_global_variables import language

labels = language.values['workflow_win']['edit_menu']
messages = language.values['workflow_win']['in_view_messages']

class OperatorMenu(object):
    """
    This class implements the manu of options for the current Hard Mesh operator.
    In can be displayed any window but needs to be connected to an HmNetworkHandler and HmOperatorsStack in order to work.
    """
    # Print messages
    VERBOSE = 0

    def __init__(self, hm_network_handler, hm_operators_stack ):
        """
        Args:
            hm_network_handler: The instance oh HmNetworkHandler to use.

        Returns:
            OperatorMenu
        """

        # === Handler Instance
        self.handler_instance = hm_network_handler

        # === Stack Instance
        self.operators_stack = hm_operators_stack


    # --------------------------------------------  UI ELEMENTS CREATION  -------------------------------------------- #
    def create_ui_elements(self):
        """
        This method create the UI elements.
        Put it in whatever window or control layout to make it appear there.
        """

        cmds.menuItem(divider = True, label='Visibility')
        cmds.menuItem(label=labels['set_visible'], c= self.set_current_as_visible)
        cmds.menuItem(label=labels['isolate_current_elements'], c = self.isolate_current_elements )
        cmds.menuItem(divider = True, label=labels['operator_sep'])
        cmds.menuItem(label=labels['delete'], c = self.handler_instance.delete_operation )
        cmds.menuItem(label=labels['bypass'], c = self.toggle_bypass )
        # cmds.menuItem(label=labels['copy_attrs'])
        # cmds.menuItem(label=labels['paste_attrs'])
        # cmds.menuItem(label=labels['move_up'], c = self.handler_instance.move_operator_up)
        # cmds.menuItem(label=labels['move_down'], c = self.handler_instance.move_operator_down)
        cmds.menuItem(divider = True, label=labels['inter_sep'])
        cmds.menuItem(label=labels['update'], c = self.update_intersections )

        if global_vars.RELEASE >= 2.5:
            cmds.menuItem(divider = True, label='Input Meshes')
            cmds.menuItem(label='Add New Mesh')
            cmds.menuItem(label='Remove Selected Mesh')

        if global_vars.RELEASE >= 3:
            cmds.menuItem(divider = True, label='Intersections')
            cmds.menuItem(label='Create Parameters Override')
            cmds.menuItem(label='Delete Parameters Override')


    # --------------------------------------------------- COMMANDS --------------------------------------------------- #
    def update_intersections(self, *args):
        """
        Call the update intersection method on the current operation
        """
        self.handler_instance.get_current_operator().update_intersections()


    def set_current_as_visible(self, *args):
        """
        Set the active handler element as visible
        """
        selected_ID = self.handler_instance.get_current_operator_ID()

        curr_ID = self.handler_instance.get_current_operator_ID()
        self.handler_instance.show_operator_result_for( curr_ID)
        # Update the icon
        item_name = self.operators_stack.items_name_list[selected_ID]
        self.operators_stack.set_visibility_icon( item_name )

        # print 'setto come visibile %s' % curr_ID


    def toggle_bypass(self, *args):
        """
        Bypass on off the current operator
        """
        curr_ID = self.handler_instance.get_current_operator_ID()
        item_name = self.operators_stack.items_name_list[curr_ID]
        start_val = self.handler_instance.get_current_operator().get_network_object().get_attribute(nc.operation_status_name)
        value = 1 if start_val else 0
        self.handler_instance.bypass_operator(curr_ID, value)

        # Update the icon
        self.operators_stack.set_bypass_icon(item_name, value)


    def isolate_current_elements(self, *args):
        """
        Isolate the current operator elements in the current active pane
        """
        def compile_current_elements_list():
            """
            Create a list of the maya object to "isolate select" in the current pane

            Args:
                None

            Returns:
                List of maya objects

            """
            out_list = []

            # Get elements of current operator
            operator = self.handler_instance.get_current_operator()

            net_obj = operator.get_network_object()
            in_meshes = operator.get_input_meshes()

            # Add net to list
            out_list.append(net_obj.get_name() )

            # Add in meshes to list
            for mesh in in_meshes:
                out_list.append(mesh.get_name() )

            return out_list


        selected_ID = self.handler_instance.get_current_operator_ID()


        obj_to_isolate = compile_current_elements_list()
        perspPanel = cmds.getPanel( withLabel='Persp View')

        # Turn of if is already on
        if cmds.isolateSelect(perspPanel, q=True, state=True):
            cmds.isolateSelect( perspPanel, state=False )

            # Set the isolation mode unactive on the operator stack
            self.handler_instance.get_current_operator().get_network_name()
            self.operators_stack.update_isolated_operator(False)

            # Re activate the update of the stack
            self.operators_stack.UPDATE_ON_CLICK = True

            # Prompt the message
            cmds.inViewMessage( amg=messages['isolate_deactivated'], pos='midCenter', fade=True )


        # Turn on the isolation
        else:
            cmds.isolateSelect( perspPanel, state=True )

            for obj in obj_to_isolate:
                cmds.isolateSelect( perspPanel, addDagObject = obj )

            # If the current is not visible, turn it on and prompt a message
            if not self.handler_instance.get_current_operator().get_result_mesh().get_attribute('visibility'):
                self.handler_instance.show_operator_result_for( selected_ID )


                # Update the icon
                item_name = self.operators_stack.items_name_list[selected_ID]
                self.operators_stack.set_visibility_icon( item_name )


            # Set the isolation mode active on the operator stack
            self.operators_stack.update_isolated_operator(True)

            # Deactivate the update of the stack
            self.operators_stack.UPDATE_ON_CLICK = False

            # Prompt the message
            net_name = self.handler_instance.get_current_operator().get_network_name()
            cmds.inViewMessage( amg=messages['isolate_activated'].format(net_name), pos='midCenter', fade=True )



